package ExemplesJDBC.GererPrets;
import java.sql.*;
import java.util.*;

// Faade pour le cas d'utilisation EnregistrerPrets
// Isole la classe de contrle des dtails de la gestion de persistence par courtier
// NB Les mthodes de la faade doivent tre appeles en squence
public class FacadeEnregistrerPrets 
{
  private Connection uneConnection;
  private Utilisateur unUtilisateur;
  private Exemplaire unExemplaire;
  private CourtierBDPrtEnCours unCourtierBDPrtEnCours;
  public FacadeEnregistrerPrets()throws Exception
  {
      // Le constructeur cre la Connection globale pour la faade
      UsineConnection uneUsineConnection = new UsineConnection();
      uneConnection = uneUsineConnection.getConnectionSansAutoCommit(
            "oracle.jdbc.driver.OracleDriver",
            "jdbc:oracle:thin:@localhost:1521:orcl",
            "clerat","oracle");
  }
  
  // Cherche un Utilisateur avec ses PretsEnCours et retourne
  // les donnes ncessaires pour EnregistrerPrets (OTDUtilisateurPrets)
  // Une exception est leve si l'Utilisateur n'existe pas
  public OTDUtilisateurPrets chercherOTDUtilisateurPrets(String idUtilisateur)throws Exception {
  
    // Cration du courtier et dmatrialisation de l'Utilisateur
    CourtierBDUtilisateur unCourtierBDUtilisateur = new CourtierBDUtilisateur(uneConnection);
    unUtilisateur = 
        unCourtierBDUtilisateur.chercherUtilisateurParIdUtilisateur(idUtilisateur);
        
    //Cration du courtier et dmatrialisation des PrtsEnCours
    unCourtierBDPrtEnCours = new CourtierBDPrtEnCours(uneConnection);
    unCourtierBDPrtEnCours.chercherLesPrtsEnCours(unUtilisateur);

    // Retourner l'objet de transfert de donnes OTDUtilisateurPrets
    if (unUtilisateur instanceof Membre){
      unCourtierBDUtilisateur.chercherVariablesStatiquesDeMembre();
      Membre unMembre = (Membre)unUtilisateur;
      return new OTDUtilisateurPrets(
                  unMembre.getCatgorie(),
                  unMembre.conditionsPrtAcceptes(),
                  unMembre.getNbPrtsEnCours(),
                  Membre.getNbMaxPrts(),
                  unMembre.getNbRetards());
    } else {
      return new OTDUtilisateurPrets(
        unUtilisateur.getCatgorie(),true,unUtilisateur.getNbPrtsEnCours(),0,0);
    }
  }

  public String getStatutExemplaire(String idExemplaire)throws Exception{
    // Chercher l'exemplaire en passant par le courtier et retourner le statut
    CourtierBDExemplaire unCourtierBDExemplaire = new CourtierBDExemplaire(uneConnection);
    unExemplaire = unCourtierBDExemplaire.chercherExemplaireParIdExemplaire(idExemplaire);
    return unExemplaire.getStatut();
  }

  // Insre le prt et retourne la date d'enregistrement
  // NB Suppose que unUtilisateur et unExemplaire ont t dmatrialiss
  public java.sql.Date insererPretEnCours() throws Exception{
    Calendar maintenant = Calendar.getInstance(); // Calendrier avec date actuelle
    java.sql.Date dateMaintenant = new java.sql.Date(maintenant.getTime().getTime());
    PrtEnCours leNouveauPrtEnCours = 
          new PrtEnCours(unUtilisateur,dateMaintenant,unExemplaire);

    // Matrialiser le PrtEnCours dans la BD
    unCourtierBDPrtEnCours.insrerPrtEnCours(leNouveauPrtEnCours);
    return dateMaintenant;
  }

  // Confirmer la transaction et fermer la Connection
  public void confirmerTransaction()throws Exception{
    uneConnection.commit();
    uneConnection.close();
  }
}
